#version 150

in vec2 fragTexCoord;
in vec3 l;
in vec3 n;
in vec3 v;

uniform bool texturedColor;
uniform bool texturedNormal, texturedHeight;
uniform vec3 mat_ambient, mat_diffuse, mat_specular;
uniform vec3 light_ambient, light_diffuse, light_specular;
uniform float mat_shininess;
uniform float scale, bias;

uniform sampler2D texColor;
uniform sampler2D texNormal;
uniform sampler2D texHeight;

out vec4 out_Color;

void main()
{

	if(texturedColor == false)
	{
		out_Color = vec4(1.0,0.0,0.0,1.0);
	}
	else
	{
		
		// texturedNormal == true, then do normal mapping
		if(texturedNormal)
		{
			
			vec2 newTexCoord;
			vec3 V = normalize(v);

			// height texture for offset limited parallax mapping here?
			if(texturedHeight)
			{

				// get height from texture
				float height = texture2D(texHeight, fragTexCoord).r*3.0f-1.5f;

				// compute new height using uniform variables
				height = height * -0.01 + 0.003;
				//height = height * scale - bias;
				//height = height * -0.03f + 0.015f;
				

				// translate texture coordinates along surface corresponding to the computed height
				newTexCoord = fragTexCoord + (height * V.xy);
			}else{

				// no height map here -> classical bump mapping, just get normal map
				newTexCoord = fragTexCoord;
			}

			// get/compute normal-, light- and reflection vectors
			vec3 N = normalize(texture2D(texNormal, newTexCoord).rgb*2.0f-1.0f);
			vec3 L = normalize(l);
			vec3 R = reflect(L,N);

			// define light attenuation factor
			float attenFac = max(0.0, 2.3f*dot(L, L));

			// dot product of N and L
			float NdotL = max(0.0, dot(N, L));

			// dot product of R and V
			float VdotR = dot(V,R);
			
			// define ambient, diffuse and specular terms of our light model
			vec4 ambient = vec4(light_ambient*mat_ambient,1.0) * attenFac;
			vec4 diffuse = vec4(light_diffuse*mat_diffuse,1.0) * NdotL * attenFac;
			vec4 specular = vec4(0.0f,0.0f,0.0f,1.0f);

			// if specular reflection vector faces towards us, define specular term
			if(VdotR > 0.0f)
			{
				specular = vec4(light_specular*mat_specular,1.0f) * pow(VdotR,mat_shininess*2) * attenFac;
			}

			// combine light model terms
			vec4 color =  ambient + diffuse + specular;
    
			// texturize
			out_Color = color * texture2D(texColor, newTexCoord);
			
		}else{
			vec3 temp = light_ambient + vec3(0.3f,0.3f,0.3f);
			vec4 ambient = vec4(temp,1.0) * texture2D(texColor, fragTexCoord);
			vec4 diffuse = texture2D(texColor, fragTexCoord) * max(0.0, dot(n, l));
		 
			out_Color = ambient + diffuse;
		}
		
	}
	
}