
// Vertex shader for the scene renderer with 
// per vertex lighting and texturing


#version 150

in vec3 position;
in vec2 texCoords;
in vec3 normal;
in vec3 tangent;
in vec3 bitangent;

uniform mat4 modelMatrix, viewMatrix, projMatrix;
uniform mat3 normalMatrix;
uniform vec3 light_pos;
uniform bool texturedNormal, texturedHeight;

out vec2 fragTexCoord;
out vec3 l;
out vec3 n;
out vec3 v;

void main(void)
{
	
	fragTexCoord = texCoords;
	
	
	vec3 vertex3 = vec3(viewMatrix * modelMatrix * vec4(position, 1.0));
	vec3 light3 = vec3(viewMatrix * modelMatrix * vec4(light_pos, 1.0));

	n = normalize(normalMatrix * normal);

	if(texturedNormal || texturedHeight)
	{
		
		vec3 t = normalize (normalMatrix * tangent);
		vec3 b = cross (n, t);
		//vec3 b = normalize(bitangent);
	
		mat3 tbnMatrix = mat3( t.x, b.x, n.x,
							   t.y, b.y, n.y,
							   t.z, b.z, n.z);
		
		l = tbnMatrix * normalize(light3 - vertex3);
		v = tbnMatrix * vertex3;
		
	}
	else
	{
		l = normalize(light3 - vertex3);
		v = normalize(-vertex3);
	}
	

	gl_Position = projMatrix * viewMatrix * modelMatrix * vec4(position, 1.0);
}