// Fragment shader for the motion blur effect

#version 150


in vec2 quadTexCoords;

uniform sampler2D texColor;
uniform sampler2D texDepth;

uniform mat4 ivpMatrix;
uniform mat4 prevVpMatrix;

out vec4 out_Color;

void main(void)
{

	// init variables
	int numSamples = 10;
	float strength = 0.4f;

	// depth value at the current pixel
	float zOverW = texture2D(texDepth, quadTexCoords).r;
		
	// viewport position at this pixel in the range -1 to 1
	vec4 H = vec4((quadTexCoords.x)*2.0 - 1.0, (1-quadTexCoords.y)*2.0-1.0, zOverW, 1.0);

	// transform by the view-projection inverse
	vec4 D = ivpMatrix * H;
	
	// divide by w to get the world position
	vec4 worldPos = D/D.w;
 
	// current viewport position 
	vec4 currentPos = H; 

	// use the world position, and transform by the previous view- 
	// projection matrix
	vec4 prevPos = prevVpMatrix * worldPos; 

	// convert to nonhomogeneous points [-1,1] by dividing by w
	prevPos /= prevPos.w; 
     
	// use this frames position and last frames to compute the pixel 
	// velocity
	vec2 velocity = (currentPos - prevPos).xy/strength;
     
	// get initial color at this pixel
	vec4 color = texture2D(texColor, quadTexCoords);
	vec2 newQuadTexCoords = quadTexCoords;

	vec2 temp = newQuadTexCoords + velocity;

	if(temp.x>=0.0f && temp.x<=1.0f && temp.y>=0.0f && temp.y<=1.0f)
	{
		newQuadTexCoords += velocity;
	}
	
	// repeat for a defined number of samples
	for(int i=1; i<numSamples; ++i)
	{
		// translate texture coordinates along velocity vector
		
		temp = newQuadTexCoords + velocity;

		if(temp.x>=0.0f && temp.x<=1.0f && temp.y>=0.0f && temp.y<=1.0f)
		{
			newQuadTexCoords += velocity;
		}

		// add color at translated position to current color
		color += texture2D(texColor, newQuadTexCoords);
	}

	// output
	out_Color = color/numSamples;  
}